@@ -1,4 +1,5 @@
-#!/usr/local/bin/python2.7
+
+#!/usr/local/bin/python3.7
 # -*- coding: utf-8 -*-
 #
 # (c) Copyright 2003-2018 HP Development Company, L.P.
@@ -39,6 +40,8 @@ import math
 import os
 import sys
 import platform
+import re
+import subprocess
 from installer import core_install
 from installer import dcheck
 #from .dcheck import *
@@ -195,8 +198,12 @@ def Deskew_angle1(img):
         angle = abs(angle)
         if angle != 0:
             angles.append(angle)
+    if not angles:
+        median_angle = 0
+        return median_angle
+    else:
+        median_angle = min(angles)
 
-    median_angle = min(angles)
     for item in temp_angles:
         if median_angle == abs(item):
             median_angle = item
@@ -239,12 +246,17 @@ def Deskew_angle2(img):
         hist, score = find_score(bin_img, angle)
         scores.append(score)
  
-    best_score = max(scores)
+    if not scores:
+        best_angle = 0
+        return best_angle
+    else:
+        best_score = max(scores)
+
     best_angle = angles[scores.index(best_score)]
     if best_angle < 0 :
-        best_angle = best_angle - 1
+        best_angle = best_angle
     else:
-        best_angle = best_angle + 1
+        best_angle = best_angle
     if (best_angle >= 15 or best_angle <= -15):
         best_angle = 0
     return best_angle
@@ -321,7 +333,13 @@ def rotate_bound(image, angle):
     return cv2.warpAffine(image, M, (nW, nH))
     
 def orientangle(im):
-    from tesserocr import PyTessBaseAPI, PSM
+    if int(get_tesseract_version()) >= 4:
+        import locale
+        locale.setlocale(locale.LC_ALL, 'C')
+        from tesserocr import PyTessBaseAPI, PSM
+    else:
+        from tesserocr import PyTessBaseAPI, PSM
+
     with PyTessBaseAPI(psm=PSM.AUTO_OSD) as api:        
         try:        
             api.SetImage(im)
@@ -667,6 +685,9 @@ def adjust_color(im, factor):
     return out
    
 def merge_PDF_viewer(output,ocr):
+    if (output.endswith('.pdf') == False):
+        print("PDF viewer is trying open other than PDF file")
+        return
     pdf_viewer = ''
     pdf_viewer_list = ['kpdf', 'acroread', 'xpdf', 'evince', 'xdg-open']
     for v in pdf_viewer_list:
@@ -679,10 +700,12 @@ def merge_PDF_viewer(output,ocr):
         g = output.split(".pdf")
         output_ocr = g[0] +"_ocr.pdf"
         if sys.version_info[0] == 3:
-            cmd = "ocrmypdf" + " " + output + " " + output_ocr
+            out = subprocess.Popen(['ocrmypdf',output,output_ocr],stdout=subprocess.PIPE,stderr=subprocess.STDOUT)
+            stdout,stderr =out.communicate()
         else:
-            cmd = "pypdfocr" + "  " + output
-        os_utils.execute(cmd)
+            #cmd = "pypdfocr" + "  " + output
+            out = subprocess.Popen(['pypdfocr',output],stdout=subprocess.PIPE,stderr=subprocess.STDOUT)
+            stdout,stderr =out.communicate()
         #g = output.split(".")
         #output_ocr = g[0] +"_ocr.pdf"
         if os.path.isfile(output_ocr):
@@ -754,8 +777,12 @@ def check_skimage():
 def check_tesserocr_imutils():
     scanjet_flag = None
     try:
-        import tesserocr
-        #import imutils	
+        if int(get_tesseract_version()) >= 4:
+            import locale
+            locale.setlocale(locale.LC_ALL, 'C')
+            import tesserocr
+        else:
+            import tesserocr
     except ImportError as error:
         scanjet_flag=str(error)
 		#.split(' ')[-1]
@@ -807,10 +834,42 @@ def dominantcolor(xcord, ycord, radius, img):
     y2 =  punchhole_margin
 
     if((0 < ycord < width) and (0 < xcord < y2)):
-        s1 = (xcord+ radius)
-        s2 = (xcord+ (radius+ 8))
-        s3 = (ycord - 4)
-        s4 = (ycord + 4)
+        s1 = (xcord -4)
+        s2 = (xcord +4)
+        s3 = (ycord + (radius))
+        s4 = (ycord + (8+radius))
+        myimg = img[s3:s4, s1:s2]
+        w , h,  c = myimg.shape
+        for i in range(0, w-1):
+            for j in range(0, h-1):
+                rgblist = myimg[i,j]
+                l.append((rgblist[0],rgblist[1],rgblist[2]))
+        s1 = (xcord -4)
+        s2 = (xcord +4)
+        s3 = (ycord - (radius))
+        s4 = (ycord - (8+radius))
+        myimg = img[s3:s4, s1:s2]
+        w , h,  c = myimg.shape
+        for i in range(0, w-1):
+            for j in range(0, h-1):
+                rgblist = myimg[i,j]
+                l.append((rgblist[0],rgblist[1],rgblist[2]))
+
+    if((0 < ycord < width) and (y1 < xcord < height)):
+        s1 = (xcord -4)
+        s2 = (xcord +4)
+        s3 = (ycord + (radius))
+        s4 = (ycord + (8+radius))
+        myimg = img[s3:s4, s1:s2]
+        w , h,  c = myimg.shape
+        for i in range(0, w-1):
+            for j in range(0, h-1):
+                rgblist = myimg[i,j]
+                l.append((rgblist[0],rgblist[1],rgblist[2]))
+        s1 = (xcord -4)
+        s2 = (xcord +4)
+        s3 = (ycord - (radius))
+        s4 = (ycord - (8+radius))
         myimg = img[s3:s4, s1:s2]
         w , h,  c = myimg.shape
         for i in range(0, w-1):
@@ -819,9 +878,9 @@ def dominantcolor(xcord, ycord, radius, img):
                 l.append((rgblist[0],rgblist[1],rgblist[2]))
 
 
-    if((0 < ycord < width) and (y1 < xcord < height)):
-        s1 = (xcord- (radius+8))
-        s2 = (xcord- (radius))
+    if((x2 < ycord < width) and (0 < xcord < height)):
+        s1 = (xcord - (8+radius))
+        s2 = (xcord - (radius))
         s3 = (ycord - 4)
         s4 = (ycord + 4)
         myimg = img[s3:s4, s1:s2]
@@ -830,12 +889,10 @@ def dominantcolor(xcord, ycord, radius, img):
             for j in range(0, h-1):
                 rgblist = myimg[i,j]
                 l.append((rgblist[0],rgblist[1],rgblist[2]))
-
-    if((0 < ycord < x1) and (0 < xcord < height)):
-        s1 = (xcord -4)
-        s2 = (xcord +4)
-        s3 = (ycord + (radius))
-        s4 = (ycord + (radius+8))
+        s1 = (xcord + (8+radius))
+        s2 = (xcord + (radius))
+        s3 = (ycord - 4)
+        s4 = (ycord + 4)
         myimg = img[s3:s4, s1:s2]
         w , h,  c = myimg.shape
         for i in range(0, w-1):
@@ -843,13 +900,21 @@ def dominantcolor(xcord, ycord, radius, img):
                 rgblist = myimg[i,j]
                 l.append((rgblist[0],rgblist[1],rgblist[2]))
 
-
-
-    if((x2 < ycord < width) and (0 < xcord < height)):
-        s1 = (xcord -4)
-        s2 = (xcord +4)
-        s3 = (ycord - (radius+8))
-        s4 = (ycord - (radius))
+    if((0 < ycord < x1) and (0 < xcord < height)):
+        s1 = (xcord - (8+radius))
+        s2 = (xcord - (radius))
+        s3 = (ycord - 4)
+        s4 = (ycord + 4)
+        myimg = img[s3:s4, s1:s2]
+        w , h,  c = myimg.shape 
+        for i in range(0, w-1):
+            for j in range(0, h-1):
+                rgblist = myimg[i,j]
+                l.append((rgblist[0],rgblist[1],rgblist[2]))
+        s1 = (xcord + (8+radius))
+        s2 = (xcord + (radius))
+        s3 = (ycord - 4)
+        s4 = (ycord + 4)
         myimg = img[s3:s4, s1:s2]
         w , h,  c = myimg.shape
         for i in range(0, w-1):
@@ -894,12 +959,10 @@ def punchhole_removal(im):
     from skimage.util import img_as_ubyte
 
     ''' check for punch holes and remove  '''
-    
     max_peaks =  24 #maximum number of peaks to be found. changed from 99 to 24 for reducing the unnecessary punch holes being filled.
 
     img = np.array(im)# Load picture .
-    img_rgb = rgba2rgb(img)# convert to RGB
-    img_gray = rgb2gray(img_rgb)# convert to gray
+    img_gray = rgb2gray(img)# convert to gray
     image = img_as_ubyte(img_gray)
     width, height = image.shape
     x1 =  punchhole_margin
@@ -907,13 +970,13 @@ def punchhole_removal(im):
     y1 =  (int)(height - punchhole_margin)
     y2 =  punchhole_margin
 
-    edges = canny(image, 3, 10, 40) # perform canny to detect the edges
+    edges = auto_canny(image)
+    #edges = canny(image, 3, 10, 40) # perform canny to detect the edges
     hough_radii = np.arange(31, 34, 1) #get the radius range with step as 1.
     hough_res = hough_circle(edges, hough_radii) # detect the circles centres coordinates
 
     # Select the most prominent circles based on the max_peaks
     accums, cx, cy, radii = hough_circle_peaks(hough_res, hough_radii,total_num_peaks=max_peaks)
-
     for center_y, center_x, radius in zip(cy, cx, radii):
 
         #if the circles centres fall in the border regions, 
@@ -923,22 +986,12 @@ def punchhole_removal(im):
            ((0 < center_y < x1) and (0 < center_x < height)) or \
            ((x2 < center_y < width) and (0 < center_x < height))):
 
-            index=0
             rr, cc= circle(center_y, center_x, radius+1, img.shape)
-            dominantpix = dominantcolor(center_x, center_y, radius, img)           
-            dark_grad = [dominantpix[0], dominantpix[1],dominantpix[2]]
-            light_grad = [dominantpix[0]+1, dominantpix[1]+1, dominantpix[2]+1]
-            #white_grad = [255,255,255]
-            RGBA_list = lineargradient(dark_grad,light_grad,len(list(rr)))   
-          
+            dominantpix = dominantcolor(center_x, center_y, radius,img)          
             for i , j in zip(list(rr), list(cc)):
-                pixlist = RGBA_list[index]
-                pixtuple = tuple(pixlist)
-                img[i,j]= (pixtuple[0], pixtuple[1], pixtuple[2], 255)
-                index += 1
-           
-    finalimage=Image.fromarray(img)
+                img[i,j]= (dominantpix[0], dominantpix[1], dominantpix[2], 255)
 
+    finalimage=Image.fromarray(img)
     return finalimage
 
 def color_dropout(im,color,color_range):
@@ -1001,4 +1054,37 @@ def rgb2hsv(r, g, b):
 
     return h, s, v
 
+def check_scipy():
+    scanjet_flag = None
+    try:
+        from scipy.ndimage import interpolation as inter 	
+    except ImportError as error:
+        scanjet_flag=str(error)
+    except:
+        scanjet_flag=str("Error occurred")
+    return scanjet_flag
+
+
+def _read_string(s):
+    if sys.version_info >= (3, 0):
+        return s.decode('UTF-8')
+    else:
+        return s
+
+def get_tesseract_version():
+    try:
+        p = subprocess.Popen(['tesseract', '-v'], stderr=subprocess.PIPE, stdout=subprocess.PIPE)
+        stdout_version, version = p.communicate()
+        version = _read_string(version).strip()
+        if version == '':
+            version = _read_string(stdout_version).strip()
+        version_match = re.search(r'^tesseract ((?:\d+\.)+\d+).*', version, re.M)
+        if version_match:
+            version = version_match.group(1)
+            return version[0]
+        else:
+            return 0
+    except OSError as e:
+        #log.debug('Failed to extract tesseract version from executable: {}'.format(e))
+        return 0
 
