@@ -1,4 +1,4 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.7
 # -*- coding: utf-8 -*-
 #
 # (c) Copyright 2003-2015 HP Development Company, L.P.
@@ -115,6 +115,7 @@ deskew_image = False
 lineart_mode = False
 document_merge = False
 mixed_feed = False
+back_side = False
 batchsepBC = False
 batchsepBP = False
 barcode = False
@@ -131,6 +132,7 @@ orient = 0
 orient_list = []
 multipick_error_message = "The scan operation has been cancelled or a multipick or paper is jammed in the ADF.\nIf you cancelled the scan,click OK.\nIf the scan was terminated due to a multi-feed or paper jam in the ADF,\ndo the following:\n\n1)Clear the ADF path. For instructions see your product documentation.\n2)Check the sheets are not stuck together. Remove any staples, sticky notes,tape or other objects.\n3)Restart the scan\n\nNote:If necessary, turn off automatic detection of multi-pick before starting a new scan\n"
 SANE_STATUS_MULTIPICK=12
+SANE_STATUS_JAMMED=6
 
 PAGE_SIZES = { # in mm
     '5x7' : (127, 178, "5x7 photo", 'in'),
@@ -206,10 +208,6 @@ def createPagesFile(adf_page_files,pages_file,file_type='.png'):
         for p in adf_page_files:
             im = Image.open(p)
             output = utils.createBBSequencedFilename(pages_file, file_type, output_path)
-            '''pyPlatform = platform.python_version()
-            num = pyPlatform.split('.')
-            if num[0] >= '3':
-                im = im.convert("RGB")'''
             try:
                 im.save(output,compress_level=1,quality=55)
             except:
@@ -321,32 +319,7 @@ try:
         ("[OPTIONS] (advanced)", "", "header", False),
         ("Set the scanner compression mode:", "-x<mode> or --compression=<mode>, <mode>='raw', 'none' or 'jpeg' ('jpeg' is default) ('raw' and 'none' are equivalent)", "option", False),]
 
-    '''scanjet=[utils.USAGE_SPACE,("[OPTIONS] (Scanjet)", "", "header", False),("multi-pick mode:", "--multipick (multipick detection enabled)", "option", False),
-        ("Auto-orientation mode:", "--autoorient (auto orientation enabled)", "option", False),
-        ("Crushed:", "--crushed (crushed enabled)", "option", False),
-        ("Background Color Removal:", "--bg_color_removal (Background Color Removal(make white) enabled)", "option", False),
-        ("Punch Hole Removal:", "--punchhole_removal (Background Color Removal(make white) enabled)", "option", False),
-        ("batchsep and blankpage mode:", "--batchsepBP (Batch Seperation Blankpage enabled)", "option", False),
-        ("batchsep and Barcode mode:", "--batchsepBC (Batch Seperation barcode enabled)", "option", False),
-        ("Auto-Crop mode:", "--autocrop (auto crop enabled)", "option", False),
-	("Deskew mode:", "--deskew (deskew enabled)", "option", False),
-	("Mixed-feed mode:", "--mixedfeed (multi feed enabled)", "option", False),
-        ("Document merge:", "--docmerge (document merge enabled)", "option", False),
-        ("Document merge ADF Flatbed:", "--adf_fladbed_merge (document merge ADF and Flatbed UI application enabled)", "option", False),
-        ("Color Dropout Red :", "-color_dropout_red_value=<color_dropout_red_value> or --color_dropout_red_value=<color_dropout_red_value>", "option", False),
-        ("Color Dropout Green :", "-color_dropout_green_value=<color_dropout_green_value> or --color_dropout_green_value=<color_dropout_green_value>", "option", False),
-        ("Color Dropout Blue :", "-color_dropout_blue_value=<color_dropout_blue_value> or --color_dropout_blue_value=<color_dropout_blue_value>", "option", False),
-        ("Color Dropout Range :", "-color_range=<color_range> or --color_range=<color_range>", "option", False),
-        ("", "Example:color_dropout = [0,0,0,0].", "option", False),
-        ("Image contrast:", "--contrast=<contrast>", "option", False),
-        ("", "The contrast range varies from device to device.", "option", False),
-        ("Image brightness:", "--brightness=<brightness>", "option", False),
-        ("", "The brightness range varies from device to device.", "option", False),
-        ("Image sharpness:", "--sharpness=<sharpness>", "option", False),
-        ("", "The sharpness range varies from device to device.", "option", False),
-        ("Image color:", "--color_value=<color_value>", "option", False),
-        ("", "The color range varies from device to device.", "option", False),
-	("Blank Page Removal:", "--blankpage(blank page removal enabled)", "option", False)]'''
+    
     scan_parseStdOpts = ['dest=', 'mode=', 'res=', 'resolution=',
                           'resize=', 'adf', 'duplex', 'dup', 'unit=',
                           'units=', 'area=', 'box=', 'tlx=',
@@ -361,61 +334,8 @@ try:
                           'subject=', 'to=', 'from=', 'jpg',
                           'grey-scale', 'gray-scale', 'about=',
                           'editor=', 'dp=', 'dest-printer=', 'dd=',
-                          'dest-device=', 'brightness=', 'contrast=','filetype=', 'path=', 'uiscan', 'sharpness=','color_dropout_red_value=','color_dropout_green_value=','color_dropout_blue_value=','color_range=', 'color_value=','multipick','autoorient','blankpage','batchsepBP','mixedfeed', 'crushed', 'bg_color_removal','punchhole_removal','docmerge','adf_fladbed_merge','batchsepBC','deskew','autocrop',]
+                          'dest-device=', 'brightness=', 'contrast=','filetype=', 'path=', 'uiscan', 'sharpness=','color_dropout_red_value=','color_dropout_green_value=','color_dropout_blue_value=','color_range=', 'color_value=','multipick','autoorient','blankpage','batchsepBP','mixedfeed', 'crushed', 'bg_color_removal','punchhole_removal','docmerge','adf_fladbed_merge','batchsepBC','deskew','autocrop','backside']
 
-    '''scanjet_parseStdOpts = [ 'brightness=', 'sharpness=', 'contrast=', 'color_value=','multipick','autoorient','blankpage','batchsepBP','mixedfeed', 'crushed','docmerge','adf_fladbed_merge','batchsepBC','deskew','autocrop',]
-
-    scanjet_flag_pil=imageprocessing.check_pil()
-    scanjet_flag_numpy=imageprocessing.check_numpy()
-    scanjet_flag_opencv=imageprocessing.check_opencv()
-    scanjet_flag_tesser_utils=imageprocessing.check_tesserocr_imutils()
-    scanjet_flag_pypdf2=imageprocessing.check_pypdf2()
-    scanjet_flag_barcode=imageprocessing.check_zbar()
-    try:
-        if scanjet_flag_pil == 'PIL':
-            scanjet_parseStdOpts = []
-        if scanjet_flag_numpy == 'numpy':
-            parse_list = ['multipick','autoorient','batchsepBP','mixedfeed', 'crushed','docmerge','adf_fladbed_merge','batchsepBC','deskew','autocrop']
-            for thing in parse_list: 
-                try:
-                    scanjet_parseStdOpts.remove(thing)
-                except ValueError:
-                    print ('')
-        if scanjet_flag_opencv == 'cv2':
-            parse_list = ['deskew','autocrop','mixedfeed']
-            for thing in parse_list: 
-                try:
-                    scanjet_parseStdOpts.remove(thing)
-                except ValueError:
-                    print ('')
-        if scanjet_flag_tesser_utils == 'tesserocr' or scanjet_flag_tesser_utils == 'imutils':
-            parse_list = ['autoorient']
-            for thing in parse_list: 
-                try:
-                    scanjet_parseStdOpts.remove(thing)
-                except ValueError:
-                    print ('')
-        if scanjet_flag_pypdf2 == 'PyPDF2':
-            parse_list = ['docmerge','adf_fladbed_merge']
-            for thing in parse_list: 
-                try:
-                    scanjet_parseStdOpts.remove(thing)
-                except ValueError:
-                    print ('')
-        if scanjet_flag_barcode == 'False':
-            parse_list = ['batchsepBC']
-            for thing in parse_list: 
-                try:
-                    scanjet_parseStdOpts.remove(thing)
-                except ValueError:
-                    print ('')
-    except TypeError:
-        print ('')'''
-    #if con_device == 5000 or con_device == 7500:
-	#scanjet=[("Destination:", "--path=<destination>", "option", False),]
-        #extra_options.extend(scanjet)
-        #scanjet_parseStdOpts = ['filetype=', 'path=', 'uiscan', 'sharpness=', 'color_value=','multipick','autoorient','blankpage','batchsepBP','mixedfeed', 'crushed','docmerge','adf_fladbed_merge','batchsepBC','deskew','autocrop',] 
-        #scan_parseStdOpts.extend(scanjet_parseStdOpts)
     mod.setUsage(module.USAGE_FLAG_DEVICE_ARGS, extra_options, see_also_list=[])
 
     #print devicelist 
@@ -899,6 +819,13 @@ try:
             except ValueError:
                 log.error("Invalid Option.Using default of False")
                 mixed_feed = False
+        elif o == '--backside':
+            try:
+                back_side = True
+                duplex = True
+            except ValueError:
+                log.error("Invalid Option.Using default of False")
+                back_side = False
         elif o == '--docmerge':
             try:
                 document_merge = True                
@@ -1003,15 +930,16 @@ try:
             sys.exit(1)
 
         sane.init()
-        devices = sane.getDevices()
+		# Commenting redundant getDevices() call since device list are already fetched in the beginning
+        #devices = sane.getDevices()
 
         # Make sure SANE backend sees the device...
-        for d, mfg, mdl, t in devices:
-            if d == device_uri:
-                break
-        else:
-            log.error("Unable to locate device %s using SANE backend hpaio:. Please check HPLIP installation." % device_uri)
-            sys.exit(1)
+        #for d, mfg, mdl, t in devices:
+        #    if d == device_uri:
+        #        break
+        #else:
+        #    log.error("Unable to locate device %s using SANE backend hpaio:. Please check HPLIP installation." % device_uri)
+        #    sys.exit(1)
 
         if uiscan == False:
             log.info(log.bold("Using device %s" % device_uri))
@@ -1023,6 +951,9 @@ try:
             if multipick and e.args[0] == SANE_STATUS_MULTIPICK:
                  log.error(multipick_error_message)
                  sys.exit(2)
+            if e.args[0] == SANE_STATUS_JAMMED:
+                 log.error(multipick_error_message)
+                 sys.exit(7)
             sane.reportError(e.args[0])
             sys.exit(1)
 
@@ -1041,8 +972,10 @@ try:
              adf = True
         elif len(source_option) == 3 and ('ADF-SinglePage' in source_option) and ('ADF-MultiPage-Simplex' in source_option) and ('ADF-MultiPage-Duplex' in source_option):
              log.debug("Device has only ADF support")
-             adf = True 
-
+             adf = True
+        elif len(source_option) == 2 and ('ADF' in source_option) and ('Duplex' in source_option):
+             log.debug("Device has only ADF support")
+             adf = True
         if adf:
             try:
                 if ('ADF' not in source_option) and ('ADF-SinglePage' not in source_option) and ('ADF-MultiPage-Simplex' not in source_option) and ('ADF-MultiPage-Duplex' not in source_option) and ('ADF Simplex' not in source_option) and ('ADF Duplex' not in source_option):
@@ -1085,6 +1018,12 @@ try:
             except scanext.error:
                 log.debug("Error setting source or batch-scan option (this is probably OK).")
 
+        if multipick and (not re.search(r'_2000_s2', device_uri)) : 
+            MPICK = 1
+            device.setOption("multi-pick", int(MPICK))
+        else: 
+            MPICK = 0
+            device.setOption("multi-pick", int(MPICK))
 
         tlx = device.getOptionObj('tl-x').limitAndSet(tlx)
         tly = device.getOptionObj('tl-y').limitAndSet(tly)
@@ -1308,6 +1247,7 @@ try:
 
         no_docs = False
         page = 1
+        backpage_count = 1
         barcode_index=0
         blankpage_index=0
         adf_page_files = []
@@ -1336,6 +1276,9 @@ try:
                         if adf and e.args[0] == SANE_STATUS_MULTIPICK and multipick:
                             log.error(multipick_error_message)
                             sys.exit(2)
+                        if adf and (e.args[0] == SANE_STATUS_JAMMED) :
+                            log.error(multipick_error_message)
+                            sys.exit(7)
                         sane.reportError(e.args[0])
                         sys.exit(1)
                     except KeyboardInterrupt:
@@ -1356,6 +1299,9 @@ try:
                         if multipick:
                             log.error(multipick_error_message)
                             sys.exit(2)
+                    if adf and status == SANE_STATUS_JAMMED:
+                        log.error(multipick_error_message)
+                        sys.exit(7)
 
                     if expected_bytes > 0:
                         if adf:
@@ -1386,8 +1332,8 @@ try:
                                                 utils.format_bytes(bytes_read))
 
                                 if status != scanext.SANE_STATUS_GOOD:
-                                    if status == SANE_STATUS_MULTIPICK and multipick:
-                                        log.error("ADF_MPD multipick error %d" % (status))
+                                    if (status == SANE_STATUS_MULTIPICK and multipick) or (status == SANE_STATUS_JAMMED):
+                                        log.error("ADF_MPD multipick or Jam error %d" % (status))
                                         log.error("Error in reading data. Status=%d " % (status))
 					#sys.exit(2)
 					
@@ -1456,182 +1402,190 @@ try:
                         except ValueError:
                             log.error("Did not read enough data from scanner (I/O Error?)")
                             sys.exit(1)
-                    #if blank_page:
-                    isBlankPage = imageprocessing.blankpage(im,lineart_mode)
-                    
-                    if document_merge and duplex and blank_page:   
-                        if isBlankPage:
-                            if blank_cnt == 0:
-                                if page%2 != 0:
-                                    blank_cnt += 1
-                                    page_list.append(page)
-                            else:
-                                if page-1 in page_list:
-                                    blank_cnt += 1
-                                else:
+                    if uiscan == True and back_side and backpage_count%2 != 0:
+                        pass
+                    else:       
+
+                        #if blank_page:
+                        isBlankPage = imageprocessing.blankpage(im,lineart_mode)
+                        
+                        if document_merge and duplex and blank_page:   
+                            if isBlankPage:
+                                if blank_cnt == 0:
                                     if page%2 != 0:
-                                        blank_cnt = 1
-                                        page_list[:]
-                                        page_list.append(page)			  
-                    if blank_page and isBlankPage:
-                        if adf:
-                            if batchsepBP:
+                                        blank_cnt += 1
+                                        page_list.append(page)
+                                else:
+                                    if page-1 in page_list:
+                                        blank_cnt += 1
+                                    else:
+                                        if page%2 != 0:
+                                            blank_cnt = 1
+                                            page_list[:]
+                                            page_list.append(page)			  
+                        if blank_page and isBlankPage:
+                            if adf:
+                                if batchsepBP:
+                                    blankpage_found=1
+                                    blankpage_count=blankpage_count+1
+                                    blankpage_index=blankpage_index+1
+                                    if page == 1:
+                                        blankpage_first_page = True
+                                if not (document_merge and duplex): 
+                                    page += 1
+                                    continue
+                            else:
+                                sys.exit(0)
+                        elif isBlankPage:
+                            if adf and batchsepBP:
                                 blankpage_found=1
                                 blankpage_count=blankpage_count+1
                                 blankpage_index=blankpage_index+1
                                 if page == 1:
                                     blankpage_first_page = True
-                            if not (document_merge and duplex): 
-                                page += 1
-                                continue
-                        else:
-                            sys.exit(0)
-                    elif isBlankPage:
-                        if adf and batchsepBP:
-                            blankpage_found=1
-                            blankpage_count=blankpage_count+1
-                            blankpage_index=blankpage_index+1
-                            if page == 1:
-                                blankpage_first_page = True
-                    #if crushed:
-                        #im = imageprocessing.crushed(im)
-                    if deskew_image and (isBlankPage == False):
-                        if adf:
-                            im = imageprocessing.deskew(im)
-                        else:
+                        #if crushed:
+                            #im = imageprocessing.crushed(im)
+                        if deskew_image and (isBlankPage == False):
+                            if adf:
+                                im = imageprocessing.deskew(im)
+                            else:
+                                #im = imageprocessing.autocrop(im)
+                                im = imageprocessing.deskew(im) 
+                        #if mixed_feed:
+                            #im = imageprocessing.mixedfeed(im)
+                        if auto_crop and (isBlankPage == False):
                             im = imageprocessing.autocrop(im)
-                            im = imageprocessing.deskew(im) 
-                    if mixed_feed:
-                        im = imageprocessing.mixedfeed(im)
-                    if auto_crop and (isBlankPage == False):
-                        im = imageprocessing.autocrop(im)
-                    if auto_orient:
-                        if not isBlankPage:
-                            orient = imageprocessing.orientangle(im)
-                            orient_list.append(orient)                                                     
-                            im = imageprocessing.autoorient(im, orient)
-                        else:
-                            orient_list.append(0)
-                    if uiscan == True and set_brightness:
-                        factor = brightness/100
-                        #print factor
-                        im = imageprocessing.adjust_brightness(im, factor)
-                    if uiscan == True and set_contrast:
-                        factor = contrast/100
-                        #print factor
-                        im = imageprocessing.adjust_contrast(im, factor)
-                    if set_sharpness:
-                        factor = sharpness/100
-                        #print factor
-                        im = imageprocessing.adjust_sharpness(im, factor)  
-                    if set_color_value:
-                        factor = color_value/100
-                        #print factor
-                        im = imageprocessing.adjust_color(im, factor)  
-                    pyPlatform = platform.python_version()
-                    num = pyPlatform.split('.')       					
-                    if batchsepBC and num[0] < '3':
-                        import zbar
-                        scanner = zbar.ImageScanner()
-                        scanner.parse_config('enable')
-                        log.debug("Here in barcode detection")
+                        if auto_orient:
+                            if not isBlankPage:
+                                orient = imageprocessing.orientangle(im)
+                                orient_list.append(orient)                                                     
+                                im = imageprocessing.autoorient(im, orient)
+                            else:
+                                orient_list.append(0)
+                        if uiscan == True and set_brightness:
+                            factor = brightness/100
+                            #print factor
+                            im = imageprocessing.adjust_brightness(im, factor)
+                        if uiscan == True and set_contrast:
+                            factor = contrast/100
+                            #print factor
+                            im = imageprocessing.adjust_contrast(im, factor)
+                        if set_sharpness:
+                            factor = sharpness/100
+                            #print factor
+                            im = imageprocessing.adjust_sharpness(im, factor)  
+                        if set_color_value:
+                            factor = color_value/100
+                            #print factor
+                            im = imageprocessing.adjust_color(im, factor)  
+                        pyPlatform = platform.python_version()
+                        num = pyPlatform.split('.')       					
+                        if batchsepBC and num[0] < '3':
+                            import zbar
+                            scanner = zbar.ImageScanner()
+                            scanner.parse_config('enable')
+                            log.debug("Here in barcode detection")
 			
-                        bar_image = im.convert('L')
+                            bar_image = im.convert('L')
 
-                        width, height = bar_image.size
-                   
-                        raw_bar = bar_image.tobytes()
+                            width, height = bar_image.size
+                       
+                            raw_bar = bar_image.tobytes()
 
-                        my_stream = zbar.Image(width, height, 'Y800', raw_bar)
-                        scanner.scan(my_stream)
-                    
-                        #if barcode and batchsep:
-                        for symbol in my_stream:
-                            #print 'decoded', symbol.type, 'symbol', '"%s"' % symbol.data
-                            if symbol.data!='':
-                                barcode_found=1
-                                barcode_data.append(symbol.data)
-                                barcode_count=barcode_count+1
-                                barcode_index=barcode_index+1
-                                if page == 1:
-                                    barcode_first_page = True
-                                break;
-                            else:
-                                barcode_found=0
+                            my_stream = zbar.Image(width, height, 'Y800', raw_bar)
+                            scanner.scan(my_stream)
+                        
+                            #if barcode and batchsep:
+                            for symbol in my_stream:
+                                #print 'decoded', symbol.type, 'symbol', '"%s"' % symbol.data
+                                if symbol.data!='':
+                                    barcode_found=1
+                                    barcode_data.append(symbol.data)
+                                    barcode_count=barcode_count+1
+                                    barcode_index=barcode_index+1
+                                    if page == 1:
+                                        barcode_first_page = True
+                                    break;
+                                else:
+                                    barcode_found=0
 
-                    if punchhole_removal:
-                        im = imageprocessing.punchhole_removal(im)
-                    if set_color_dropout:
-                        im = imageprocessing.color_dropout(im,[color_dropout_red,color_dropout_green,color_dropout_blue],color_range_value)
-                    if bg_color_removal:
-                        im = imageprocessing.bg_color_removal(im)
-                    if crushed:
-                        im = imageprocessing.crushed(im)
+                        if punchhole_removal:
+                            im = imageprocessing.punchhole_removal(im)
+                        if set_color_dropout:
+                            im = imageprocessing.color_dropout(im,[color_dropout_red,color_dropout_green,color_dropout_blue],color_range_value)
+                        if bg_color_removal:
+                            im = imageprocessing.bg_color_removal(im)
+                        if crushed:
+                            im = imageprocessing.crushed(im)
 
-                    if uiscan == True:
-                        if adf:
-                            if (save_file == 'pdf'):
-                                if (not (document_merge and duplex and save_file == 'pdf')) or (imageprocessing.check_pypdf2() == None):
+                        if uiscan == True:
+                            if adf:
+                                if (save_file == 'pdf'):
+                                    if (not (document_merge and duplex and save_file == 'pdf')) or (imageprocessing.check_pypdf2() == None):
+                                        #ext = ".png"
+                                        im = im.convert("RGB")
+                                if barcode_count>0:
+                                    if barcode_first_occurence == True:
+                                        if barcode_first_page == False:
+                                            createPagesFile(adf_page_files,'hpscan', ext)
+                                        barcode_first_occurence = False
+                                    else:
+                                        createPagesFile(adf_page_files,barcode_data[len(barcode_data)-2], ext)                                    
+                                    barcode_count=barcode_count-1
+                                    del adf_page_files[:]
+                                if blankpage_count>0:
+                                    if blankpage_first_occurence == True:
+                                        if blankpage_first_page == False:
+                                            createPagesFile(adf_page_files,'hpscan', ext)
+                                        blankpage_first_occurence = False
+                                    else:
+                                        createPagesFile(adf_page_files,"batchSep_00%d"%bp_no, ext)
+                                    blankpage_count=blankpage_count-1
+                                    bp_no += 1
+                                    del adf_page_files[:]
+                                '''if (save_file == 'pdf'):
                                     #ext = ".png"
+                                    im = im.convert("RGB")'''
+                                if merge_ADF_Flatbed == True and save_file == 'pdf':
+                                    temp_output = utils.createSequencedFilename("hpscanMerge", ext,output_path)
+                                else:
+                                    if (document_merge and duplex and save_file == 'pdf') or (imageprocessing.check_pypdf2() != None):
+                                        temp_output = utils.createSequencedFilename("hpscan", '.png', output_path)
+                                    else:
+                                        temp_output = utils.createSequencedFilename("hpscan",ext, output_path)
+                                adf_page_files.append(temp_output)
+                                #print "entered flatbed save"
+                                '''pyPlatform = platform.python_version()
+                                num = pyPlatform.split('.')
+                                if num[0] >= '3':
+                                    im = im.convert("RGB")'''                           
+                                try:
+                                    im.save(temp_output,compress_level=1,quality=55)
+                                except:
                                     im = im.convert("RGB")
-                            if barcode_count>0:
-                                if barcode_first_occurence == True:
-                                    if barcode_first_page == False:
-                                        createPagesFile(adf_page_files,'hpscan', ext)
-                                    barcode_first_occurence = False
-                                else:
-                                    createPagesFile(adf_page_files,barcode_data[len(barcode_data)-2], ext)                                    
-                                barcode_count=barcode_count-1
-                                del adf_page_files[:]
-                            if blankpage_count>0:
-                                if blankpage_first_occurence == True:
-                                    if blankpage_first_page == False:
-                                        createPagesFile(adf_page_files,'hpscan', ext)
-                                    blankpage_first_occurence = False
-                                else:
-                                    createPagesFile(adf_page_files,"batchSep_00%d"%bp_no, ext)
-                                blankpage_count=blankpage_count-1
-                                bp_no += 1
-                                del adf_page_files[:]
-                            '''if (save_file == 'pdf'):
-                                #ext = ".png"
-                                im = im.convert("RGB")'''
-                            if merge_ADF_Flatbed == True and save_file == 'pdf':
-                                temp_output = utils.createSequencedFilename("hpscanMerge", ext,output_path)
-                            else:
-                                if (document_merge and duplex and save_file == 'pdf') or (imageprocessing.check_pypdf2() != None):
-                                    temp_output = utils.createSequencedFilename("hpscan", '.png', output_path)
-                                else:
-                                    temp_output = utils.createSequencedFilename("hpscan", ext, output_path)
-                            adf_page_files.append(temp_output)
-                            #print "entered flatbed save"
-                            '''pyPlatform = platform.python_version()
-                            num = pyPlatform.split('.')
-                            if num[0] >= '3':
-                                im = im.convert("RGB")'''                           
-                            try:
-                                im.save(temp_output,compress_level=1,quality=55)
-                            except:
-                                im = im.convert("RGB")
-                                im.save(temp_output,compress_level=1,quality=55)
-                            '''if (save_file == 'pdf'):
-                                ext = ".pdf"'''        
-                            if document_merge and duplex and blank_page:
-                                if blank_cnt == 2:
-                                    os.unlink(adf_page_files.pop())
-                                    os.unlink(adf_page_files.pop())
-                                    blank_cnt = 0
-                                    page_list[:]
+                                    im.save(temp_output,compress_level=1,quality=55)
+                                '''if (save_file == 'pdf'):
+                                    ext = ".pdf"'''        
+                                if document_merge and duplex and blank_page:
+                                    if blank_cnt == 2:
+                                        os.unlink(adf_page_files.pop())
+                                        os.unlink(adf_page_files.pop())
+                                        blank_cnt = 0
+                                        page_list[:]
 
-                    elif uiscan == False:
-                        if adf or output_type == 'pdf':
-                            temp_output = utils.createSequencedFilename("hpscan_pg%d_" % page, ".png")
-                            adf_page_files.append(temp_output)
-                            im.save(temp_output,compress_level=1,quality=55)
+                        elif uiscan == False:
+                            if adf or output_type == 'pdf':
+                                temp_output = utils.createSequencedFilename("hpscan_pg%d_" % page, ".png")
+                                adf_page_files.append(temp_output)
+                                im.save(temp_output,compress_level=1,quality=55)
                 elif uiscan == True and status == scanext.SANE_STATUS_MULTIPICK and multipick:
                     log.error("ADF_MPD multipick error %d" % (status))
                     log.error("Error in reading data. Status=%d bytes_read=%d." % (status, bytes_read))
                     sys.exit(2)
+                elif uiscan == True and (status == SANE_STATUS_JAMMED):
+                    log.error("ADF_MPD multipick or Jam error %d" % (status))
+                    log.error("Error in reading data. Status=%d bytes_read=%d." % (status, bytes_read))
+                    sys.exit(7)
                 else:
                     log.error("No data read.")
                     sys.exit(1)
@@ -1640,6 +1594,7 @@ try:
                     break
                 
                 page += 1
+                backpage_count += 1
             #print "*** Total Time Taken \n"
             #print datetime.now()-start
         finally:
@@ -1718,17 +1673,23 @@ try:
                         output = utils.createSequencedFilename("hpscanMerge", ext,output_path)
                     else:
                         output = utils.createSequencedFilename("hpscan", ext,output_path)'''
-                if len(adf_page_files) > 1:
+                if len(adf_page_files) > 0:
                     #print "adf page files greater than 1"
                     if merge_ADF_Flatbed == True:
                         output = utils.createSequencedFilename("hpscanMerge", ext,output_path)
                     else:
                         output = utils.createSequencedFilename("hpscandoc", ext,output_path)
-                    try:      
-                        output = imageprocessing.generatePdfFile(adf_page_files,output)
-                    except ImportError:
-                        try:
+                    try:
+                        if mixed_feed:
+                            output = imageprocessing.generatePdfFile(adf_page_files,output)
+                        else:
                             output = imageprocessing.generatePdfFile_canvas(adf_page_files,output,orient_list,brx,bry,tlx,tly,output_path)
+                    except:
+                        try:
+                            if mixed_feed:
+                                output = imageprocessing.generatePdfFile_canvas(adf_page_files,output,orient_list,brx,bry,tlx,tly,output_path)                                
+                            else:
+                                output = imageprocessing.generatePdfFile(adf_page_files,output)
                         except ImportError as error:
                             if error.message.split(' ')[-1] == 'PIL':
                                 log.error("PDF output requires PIL.")
@@ -1788,9 +1749,10 @@ try:
 
             log.info("Saving to file %s" % output)
             c.save()
-            log.info("Viewing PDF file in %s" % pdf_viewer)
-            cmd = "%s %s &" % (pdf_viewer, output)
-            os_utils.execute(cmd)
+            if uiscan == True:
+                log.info("Viewing PDF file in %s" % pdf_viewer)
+                cmd = "%s %s &" % (pdf_viewer, output)
+                os_utils.execute(cmd)
             sys.exit(0)
 
         if resize != 100:
@@ -1954,7 +1916,7 @@ try:
             elif d == 'print':
                 hp_print = utils.which("hp-print", True)
                 if not hp_print:
-                    hp_print = '/usr/local/bin/python2.7 ./print.py'
+                    hp_print = '/usr/local/bin/python3.7 ./print.py'
                  
                 if dest_printer is not None:
                    cmd = '%s -p %s %s &' % (hp_print, dest_printer, output)
